package de.aes.camdroid;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Bitmap.CompressFormat;
import android.hardware.Camera;
import android.hardware.Camera.PreviewCallback;
import android.view.Display;
import android.view.Gravity;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.WindowManager.LayoutParams;
import android.widget.FrameLayout;

/**
 * Clase encargada de todas las acciones realizadas por la camara
 * @author Catrileo,Quiroz,Rodriguez
 *
 */

public class CameraView extends SurfaceView implements SurfaceHolder.Callback, PreviewCallback
{
    private byte[] mCurrentFrame;
	private Settings mSettings;
	private SurfaceHolder mHolder;
	private Camera mCamera;
    private Display mDisplay;
    private FrameLayout mOSD;
    private FrameLayout mBorder;
    private boolean isPreviewOn=false;
    private boolean isDecoding=false;
/**
 * Constructor    
 * @param context
 * @param camera
 * @param display
 * @param osd
 * @param border
 * @param settings
 */
    public CameraView (	Context context,
    					Camera camera,
				    	Display display,
				    	FrameLayout osd,
				    	FrameLayout border,
				    	Settings settings	)
    { 
    	super(context);
        mCamera = camera;
        mDisplay = display;
        mOSD = osd;
        mBorder = border;
        mSettings = settings;
        
        mHolder = getHolder();
        mHolder.addCallback(this);
        mHolder.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);
    }
/**
 * Crea el despliegue de imagen
 */
    public void surfaceCreated(SurfaceHolder holder)
    {
        try
        {
        	stopPreview();
        	mCamera.setPreviewDisplay(holder);
        }
        catch (IOException exception)
        {
            mCamera.release();
            mCamera = null;
        }
    }
/**
 * Reinicia el despliegue de imagen despues de un cambio
 */
    public void surfaceChanged(SurfaceHolder holder, int format, int w, int h)
    {
        setResolution(mSettings.PictureW(), mSettings.PictureH());
        setFrameRate(mSettings.FrameRate());
        startPreview();
    }

    public void surfaceDestroyed(SurfaceHolder holder) {}
/**
 * Inicia el despliegue de imagen    
 */
    public synchronized void startPreview()
    {
        if (!isPreviewOn)
    	{
    		isPreviewOn = true;
	    	mCamera.startPreview();
	    	notify();
    	}
    }
/**
 * Detiene el despliegue de imagen    
 */
    public synchronized void stopPreview()
    {
        if (isPreviewOn) {
	    	isPreviewOn = false;
	    	mCamera.stopPreview();
	    	notify();
    	}
    }
/**
 * Determina la resolucion    
 * @param w
 * @param h
 */
    public void setResolution(int w, int h)
    {
    	LayoutParams layoutParams;
    	Camera.Parameters camParams;
    	int x,y;
    	double camRatio = 0.98;
    	
    	x = (int)(camRatio*(double)mDisplay.getWidth());
    	y = (int)(camRatio*(double)mDisplay.getHeight());
    	layoutParams = this.calcResolution(x, y, w, h);
    	
    	layoutParams.gravity = Gravity.CENTER;
    	this.setLayoutParams(layoutParams);
    	
    	mOSD.setLayoutParams(layoutParams);
    	
    	layoutParams.gravity = Gravity.CENTER;
    	mBorder.setLayoutParams(layoutParams);
    	
    	x += (int)((1.0-camRatio)*(double)mDisplay.getHeight());
    	y += (int)((1.0-camRatio)*(double)mDisplay.getHeight());
    	layoutParams =	this.calcResolution(x, y, w, h);
    	
    	layoutParams.gravity = Gravity.CENTER;
    	mBorder.setLayoutParams(layoutParams);
    	
    	camParams = mCamera.getParameters();
    	camParams.setPreviewSize(w, h);
    	mCamera.setParameters(camParams);
    }
/**
 * Determina el framerate    
 * @param fps
 */
    public void setFrameRate(int fps)
    {
        Camera.Parameters camParams = mCamera.getParameters();
    	camParams.setPreviewFrameRate(fps);
        mCamera.setParameters(camParams);
    }
/**
 * Obtiene imagen    
 * @return 
 */
    public synchronized byte[] getPicture()
    {
    	try
    	{
    		while (!isPreviewOn) wait();
    		isDecoding = true;
    		mCamera.setOneShotPreviewCallback(this);
    		while (isDecoding)	wait();
		}
    	catch (Exception e)
    	{
    		return null;
    	}
    	return mCurrentFrame;
    }
/**
 * Calculo de la resolucion de imagen    
 * @param origWidth
 * @param origHeight
 * @param aimWidth
 * @param aimHeight
 * @return LayoutParams
 */
    private LayoutParams calcResolution (	int origWidth,
											int origHeight,
											int aimWidth,
											int aimHeight	)
	{
		double origRatio = (double)origWidth/(double)origHeight;
		double aimRatio  =  (double)aimWidth/(double)aimHeight ;
		
		if (aimRatio>origRatio)
			return new LayoutParams(origWidth,(int)(origWidth/aimRatio));
		else
			return new LayoutParams((int)(origHeight*aimRatio),origHeight);
	}
/**
 * Conversion a JPG    
 * @param rgb
 * @param raw
 * @param width
 * @param height
 */
    private void raw2jpg(int[] rgb, byte[] raw, int width, int height)
    {
	 	final int frameSize = width * height;
    	
	 	for (int j = 0, yp = 0; j < height; j++)
	 	{
    		int uvp = frameSize + (j >> 1) * width, u = 0, v = 0;
    		for (int i = 0; i < width; i++, yp++)
    		{
    			int y=0;
    			if( yp < raw.length)
    			{
    				y = (0xff & ((int) raw[yp])) - 16;
    			}

    			if (y < 0) y = 0;
    			if ((i & 1) == 0)
    			{
    				if(uvp<raw.length)
    				{
	    				v = (0xff & raw[uvp++]) - 128;
	    				u = (0xff & raw[uvp++]) - 128;
    				}
    			}
    			
    			int y1192 = 1192 * y;
    			int r = (y1192 + 1634 * v);
    			int g = (y1192 - 833 * v - 400 * u);
    			int b = (y1192 + 2066 * u);
    			
    			if (r < 0) r = 0; else if (r > 262143) r = 262143;
    			if (g < 0) g = 0; else if (g > 262143) g = 262143;
    			if (b < 0) b = 0; else if (b > 262143) b = 262143;
    			
    			rgb[yp] = 	0xff000000 | ((r << 6) & 
    						0xff0000) | ((g >> 2) &
							0xff00) | ((b >> 10) & 
							0xff);
    		}
    	}
    }
    @Override
	public synchronized void onPreviewFrame(byte[] data, Camera camera)
	{
    	int width = mSettings.PictureW() ;
		int height = mSettings.PictureH();
		
		int[] temp = new int[width*height];
		OutputStream out = new ByteArrayOutputStream();
		Bitmap bm = null;
		
		raw2jpg(temp, data, width, height);
		bm = Bitmap.createBitmap(temp, width, height, Bitmap.Config.RGB_565);
		bm.compress(CompressFormat.JPEG, mSettings.PictureQ(), out);
		mCurrentFrame = ((ByteArrayOutputStream)out).toByteArray();

    	isDecoding = false;
		notify();
	}
    
}
